//
//    FILE: dht11.cpp
// VERSION: 0.4.1
// PURPOSE: DHT11 Temperature & Humidity Sensor library for Arduino
// LICENSE: GPL v3 (http://www.gnu.org/licenses/gpl.html)
//
// DATASHEET: http://www.micro4you.com/files/sensor/DHT11.pdf
//
// HISTORY:
// George Hadjikyriacou - Original version (??)
// Mod by SimKard - Version 0.2 (24/11/2010)
// Mod by Rob Tillaart - Version 0.3 (28/03/2011)
// + added comments
// + removed all non DHT11 specific code
// + added references
// Mod by Rob Tillaart - Version 0.4 (17/03/2012)
// + added 1.0 support
// Mod by Rob Tillaart - Version 0.4.1 (19/05/2012)
// + added error codes
//

#include "dht11.h"
#include <time.h>
#include <sys/time.h>

// Return values:
// DHTLIB_OK
// DHTLIB_ERROR_CHECKSUM
// DHTLIB_ERROR_TIMEOUT

//#define PIN_IN 71
#define PIN_OUT 79



void delayMicrosecondsHard (unsigned int howLong)
{
  struct timeval tNow, tLong, tEnd ;

  gettimeofday (&tNow, NULL) ;
  tLong.tv_sec  = howLong / 1000000 ;
  tLong.tv_usec = howLong % 1000000 ;
  timeradd (&tNow, &tLong, &tEnd) ;

  while (timercmp (&tNow, &tEnd, <))
    gettimeofday (&tNow, NULL) ;
}

void delayMicroseconds (unsigned int howLong)
{
  struct timespec sleeper ;
  unsigned int uSecs = howLong % 1000000 ;
  unsigned int wSecs = howLong / 1000000 ;

  /**/ if (howLong ==   0)
    return ;
  else if (howLong  < 100)
    delayMicrosecondsHard (howLong) ;
  else
  {
    sleeper.tv_sec  = wSecs ;
    sleeper.tv_nsec = (long)(uSecs * 1000L) ;
    nanosleep (&sleeper, NULL) ;
  }
}


void dht11::initialiseEpoch (void)
{

  struct timespec ts ;

  clock_gettime (CLOCK_MONOTONIC_RAW, &ts) ;
  //epochMilli = (uint64_t)ts.tv_sec * (uint64_t)1000    + (uint64_t)(ts.tv_nsec / 1000000L) ;
  m_epochMicro = (uint64_t)ts.tv_sec * (uint64_t)1000000 + (uint64_t)(ts.tv_nsec /    1000L) ;

}

int dht11::get_humidity()
{
    return humidity;
}

int dht11::get_temperature()
{
    return temperature;
}

unsigned int dht11::micros(void)
{
    uint64_t now;
    struct timespec ts;
    clock_gettime(CLOCK_MONOTONIC_RAW, &ts);
    now = (uint64_t)ts.tv_sec*(uint64_t)1000000 + (uint64_t)ts.tv_nsec/1000;

    return (uint32_t) (now - m_epochMicro);
}

void dht11::delay(unsigned int howlong)
{
    struct timespec sleeper, dummy;
    sleeper.tv_sec = (time_t)(howlong/1000);
    sleeper.tv_nsec = (long)(howlong % 1000)*1000000;
    nanosleep(&sleeper, &dummy);
}

void dht11::ReadDHT(int dht_dpin)
{
    bool bGlobalErr = 0;
    byte dht_in;
    byte i;
    set_value(dht_dpin, (char*)"0");//low the gpio
    delay(20);

    set_value(dht_dpin, (char*)"1");//high the gpio
    delayMicroseconds(40);


    gpio_init(dht_dpin, INPUT);
    //delayMicroseconds(40);
    while(read_value(dht_dpin)==HIGH)
    {
        printf("HIGH\n");
    }
    //printf("[test:%d] read value is %d, predict is 0.\n", __LINE__,  read_value(dht_dpin));

    delayMicroseconds(80);
    while(read_value(dht_dpin) == LOW);
    delayMicroseconds(80);
    //printf("[test:%d] read value is %d, predict is 1.\n", __LINE__,  read_value(dht_dpin));

    for (i=0; i<4; i++){
        dht_dat[i] = read_dht_dat(dht_dpin);
        printf("dht_dat[%d] is %d\n", i, dht_dat[i]);
    }

    gpio_init(dht_dpin, OUTPUT);
    set_value(dht_dpin, (char*)"1");
    byte dht_check_sum = dht_dat[0]+dht_dat[1]+dht_dat[2]+dht_dat[3];
    if(dht_dat[4]!= dht_check_sum)
    {
        bGlobalErr=3;
    }
}

byte dht11::read_dht_dat(int dht_dpin)
{
  byte i = 0;
  byte result=0;
  for(i=0; i< 8; i++){
      //if(read_value(dht_dpin) == LOW)
      {
          while( read_value(dht_dpin)==LOW )
          {
              printf("holding .....................1\n");
          }
          delayMicroseconds(30);
          if(read_value(dht_dpin) == HIGH){
              result |= (1<<(7-i));
          }
          while (read_value(dht_dpin)==HIGH){
              printf("holding .....................2\n");
          }
      }
//      printf("asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf\n");
//      while( read_value(dht_dpin)==LOW );

//      delayMicroseconds(30);

//      if ( read_value(dht_dpin)==HIGH )
//        result |=(1<<(7-i));

//      while (read_value(dht_dpin)==HIGH);
    }
  return result;
}

int dht11::read(int pin)
{
	// BUFFER TO RECEIVE
	uint8_t bits[5];
	uint8_t cnt = 7;
	uint8_t idx = 0;

	// EMPTY BUFFER
	for (int i=0; i< 5; i++) bits[i] = 0;

	// REQUEST SAMPLE
    //pinMode(pin, OUTPUT);
    gpio_init(pin, OUTPUT);

    //digitalWrite(pin, LOW);
    set_value(pin, (char*)"0");                                 //LOW for 18 us
    //printf("[test] read value is %d\n", read_value(pin));
	delay(18);
    //digitalWrite(pin, HIGH);
    set_value(pin, (char*)"1");                                 //HIGH for 40us
    printf("[test] read value is %d\n", read_value(pin));
	delayMicroseconds(40);
    //pinMode(pin, INPUT);
    gpio_init(pin, INPUT);

	// ACKNOWLEDGE or TIMEOUT
	unsigned int loopCnt = 10000;
    while(read_value(pin) == LOW)
        if (loopCnt-- == 0) return DHTLIB_ERROR_TIMEOUT;        //wait for HIGH

    printf("[test] 111111111111111111111111\n");

	loopCnt = 10000;
    while(read_value(pin) == HIGH)
        if (loopCnt-- == 0) return DHTLIB_ERROR_TIMEOUT;        //wait for LOW

    printf("[test] 2222222222222222222222222222222\n");

	// READ OUTPUT - 40 BITS => 5 BYTES or TIMEOUT
	for (int i=0; i<40; i++)
	{
		loopCnt = 10000;
        while(read_value(pin) == LOW){                          //wait for HIGH
            if (loopCnt-- == 0) {
                printf("[test] 33333333333333333333333\n");
                return DHTLIB_ERROR_TIMEOUT;
            }
        }

		unsigned long t = micros();

        loopCnt = 10000;
        while(read_value(pin) == HIGH){                         //wait for LOW
            if (loopCnt-- == 0){
                printf("[test] loop 10000 , wait no LOW signal.\n");
                //return DHTLIB_ERROR_TIMEOUT;
            }
        }

		if ((micros() - t) > 40) bits[idx] |= (1 << cnt);
		if (cnt == 0)   // next byte?
		{
			cnt = 7;    // restart at MSB
			idx++;      // next byte!
		}
		else cnt--;
	}
    printf("[test] 55555555555555555555555555555\n");

	// WRITE TO RIGHT VARS
    // as bits[1] and bits[3] are allways zero they are omitted in formulas.
    humidity    = bits[0];
	temperature = bits[2]; 
    printf("humidity : %d, temperature : %d\n", humidity, temperature);

	uint8_t sum = bits[0] + bits[2];  

	if (bits[4] != sum) return DHTLIB_ERROR_CHECKSUM;
	return DHTLIB_OK;
}
//
// END OF FILE
//
