#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>
#include <string.h>

#include "gpio_common.h"
#include "eaidk310_sensor_cases.h"
#include "iic_common.h"

#define I2C_FILE_NAME "/dev/i2c-0"
#define PCF8591_ADDR 0x48

/*To start you program test:
 * 1, make sure you have root authrization.
 * 2, join up the device with gpio pin id.
 * 3, default first pin is 79, the second is 82, the third is 81.
 * 4,
 * 5, sudo ./gpio_test or ./gpio_test with root
 */

void eaidk310_sensor_pushbotton(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0; i<EAIDK310_LOOP*3; i++){
        if(read_value(pin)){
            printf("button is not pushed.\n");
        }else{
            printf("button is pushed.\n");
            printf("developer can add codes here.\n");
        }
        usleep(1000*10);
    }
}

void eaidk310_sensor_shock(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin) == HIGH)
        {
            printf("the device is not shocked.\n");
            usleep(1000);
        }
        if(read_value(pin) == LOW)
        {
            printf("the device shocked.\n");
            printf("the device active now.\n");
        }
        usleep(1000);
    }
}

void eaidk310_sensor_minireed(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin) == HIGH)
        {
            printf("the device mini reed is not magnetic inductive.\n");
            usleep(1000);
        }
        if(read_value(pin) == LOW)
        {
            printf("the device mini reed is magnetic inductive.\n");
            printf("the device active now.\n");
        }
        usleep(1000);
    }
}

//pin1 is A0, pin2 is D0
void eaidk310_sensor_reed(int pin1, int pin2)
{
    gpio_init(pin1, INPUT);
    gpio_init(pin2, OUTPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin1) == HIGH)
        {
            set_value(pin2, LOW);
            printf("the device mini reed is not magnetic inductive.\n");
            usleep(10*1000);
        }
        if(read_value(pin1) == LOW)
        {
            printf("the device mini reed is magnetic inductive.\n");
            printf("the device active now.\n");
            set_value(pin2, HIGH);
        }
        usleep(1000);
    }
}

void eaidk310_sensor_buzzer(int pin)
{
    //when this device powered on , buzzer respond to ring.
}

void eaidk310_sensor_passive_buzzer(int pin)
{
    gpio_init(pin, OUTPUT);
    int i,j;
    while(1){
        for(i=0;i<80;i++)
        {
            set_value(pin, HIGH);
            usleep(1000);
            set_value(pin, LOW);
            usleep(1000);
        }
        for(j=0;j<80;j++)
        {
            set_value(pin, HIGH);
            usleep(2000);
            set_value(pin, LOW);
            usleep(2000);
        }
    }
}

void eaidk310_sensor_twocolor_led(int pin1, int pin2)
{
    gpio_init(pin1, OUTPUT);
    gpio_init(pin2, OUTPUT);
    for(int i=0; i<EAIDK310_LOOP; i++){
        usleep(10*1000);
        set_value(pin1, HIGH);
        set_value(pin2, LOW);

        usleep(10*1000);
        set_value(pin1, LOW);
        set_value(pin2, HIGH);

        usleep(10*1000);
        set_value(pin1, HIGH);
        set_value(pin2, HIGH);

        usleep(10*1000);
        set_value(pin1, LOW);
        set_value(pin2, LOW);
        usleep(2000);
    }
}

void eaidk310_sensor_7color_flash(int pin)
{
    gpio_init(pin, OUTPUT);
    set_value(pin, HIGH);
    for(int i=0; i<EAIDK310_LOOP; i++){
        usleep(10*1000);
    }
    set_value(pin, LOW);
}

void eaidk310_sensor_laser_emit(int pin)
{
    gpio_init(pin, OUTPUT);
    set_value(pin, HIGH);
    for(int i=0; i<EAIDK310_LOOP; i++){
        usleep(10*1000);
    }
    set_value(pin, LOW);
}

void eaidk310_sensor_little_twocolor_led(int pin1, int pin2)
{
    eaidk310_sensor_twocolor_led(pin1, pin2);
}

void eaidk310_sensor_threecolor_led(int pin1, int pin2, int pin3)
{
    gpio_init(pin1, OUTPUT);
    gpio_init(pin2, OUTPUT);
    gpio_init(pin3, OUTPUT);

    for(int i=0; i<EAIDK310_LOOP; i++){
        usleep(15*1000);
        set_value(pin1, HIGH);
        set_value(pin2, LOW);
        set_value(pin3, LOW);

        usleep(15*1000);
        set_value(pin1, LOW);
        set_value(pin2, HIGH);
        set_value(pin3, HIGH);

        usleep(15*1000);
        set_value(pin1, HIGH);
        set_value(pin2, HIGH);
        set_value(pin3, LOW);

        usleep(15*1000);
        set_value(pin1, LOW);
        set_value(pin2, LOW);
        set_value(pin3, LOW);
    }
}

void eaidk310_sensor_threecolor_smd(int pin1, int pin2, int pin3)
{
    eaidk310_sensor_threecolor_led(pin1, pin2, pin3);
}

void eaidk310_sensor_relay(int pin)
{
    gpio_init(pin, OUTPUT);
    for(int i=0;i<10;i++)
    {
        set_value(pin, HIGH);
        printf("the device relay is active.\n");
        sleep(1);
        set_value(pin, LOW);
        sleep(1);

    }
}

void eaidk310_sensor_tilt_switch(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin) == HIGH)
        {
            printf("the device tilt swith is closed.\n");
            usleep(10*1000);
        }
        if(read_value(pin) == LOW)
        {
            printf("the device tilt switch is open.\n");
            printf("the device active now.\n");
        }
        usleep(1000);
    }
}

void eaidk310_sensor_hall_magnatic(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0;i<EAIDK310_LOOP*1000;i++){
        if(read_value(pin) == HIGH)
        {
            printf("the device halt magnatic is negative.\n");
            usleep(10*1000);
        }
        if(read_value(pin) == LOW)
        {
            printf("the device halt magnatic is positive.\n");
            printf("the device active now.\n");
        }
        usleep(1000);
    }
}

//pin1 is A0, pin2 is D0
void eaidk310_sensor_linear_hall(int pin1, int pin2)
{
    gpio_init(pin1, INPUT);
    gpio_init(pin2, OUTPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin1) == HIGH)
        {
            set_value(pin2, LOW);
            printf("the device linear hall is negative.\n");
//            usleep(10*1000);
        }
        if(read_value(pin1) == LOW)
        {
            printf("the device linear hall is positive.\n");
            printf("the device active now.\n");
            set_value(pin2, HIGH);
        }
        usleep(10000);
    }
}

//pin1 is A0, pin2 is D0
void eaidk310_sensor_flame(int pin1, int pin2)
{
    gpio_init(pin1, INPUT);
    gpio_init(pin2, OUTPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin1) == HIGH)
        {
            set_value(pin2, LOW);
            printf("the device flame is negative.\n");
            usleep(10*1000);
        }
        if(read_value(pin1) == LOW)
        {
            printf("the device flame is positive.\n");
            printf("the device active now.\n");
            set_value(pin2, HIGH);
        }
        usleep(10000);
    }
}

//pin1 is A0, pin2 is D0
void eaidk310_sensor_touch(int pin1, int pin2)
{
    gpio_init(pin1, INPUT);
    gpio_init(pin2, OUTPUT);
    for(int i=0;i<EAIDK310_LOOP*10000;i++){
        if(read_value(pin1) == HIGH)
        {
            set_value(pin2, LOW);
            printf("the device touch is negative.\n");
            usleep(10*1000);
        }
        if(read_value(pin1) == LOW)
        {
            printf("the device touch is positive.\n");
            printf("the device active now.\n");
            set_value(pin2, HIGH);
        }
        usleep(1000);
    }
}

void eaidk310_sensor_ball_switch(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0;i<EAIDK310_LOOP*1000;i++){
        if(read_value(pin) == HIGH)
        {
            printf("the device ball switch is negative.\n");
            usleep(100*1000);
        }
        if(read_value(pin) == LOW)
        {
            printf("the device ball switch is positive.\n");
            printf("the device active now.\n");
            usleep(10*1000);
        }
    }
}

//pin1 is A0, pin2 is D0
void eaidk310_sensor_microphone(int pin1, int pin2)
{
    gpio_init(pin1, INPUT);
    gpio_init(pin2, OUTPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin1) == LOW)
        {
            set_value(pin2, LOW);
            printf("the device microphone is negative.\n");
            usleep(1000);
        }
        if(read_value(pin1) == HIGH)
        {
            printf("the device microphone is positive.\n");
            printf("the device active now.\n");
            set_value(pin2, HIGH);
            usleep(1000);
        }
        usleep(5000);
    }

}

void eaidk310_sensor_photoresistor(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin) == LOW)
        {
            printf("the device photoresistor is negative.\n");
        }
        if(read_value(pin) == HIGH)
        {
            printf("the device photoresistor is positive.\n");
            printf("the device active now.\n");
        }
        usleep(10*1000);
    }
}

//pin1 is input, pin2 is led(output)
void eaidk310_sensor_light_cup(int pin1, int pin2)
{
    gpio_init(pin1, INPUT);
    gpio_init(pin2, OUTPUT);
    for(int i=0;i<EAIDK310_LOOP*10000;i++){
        if(read_value(pin1) == HIGH)
        {
            set_value(pin2, LOW);
            printf("the device light cup is negative.\n");
            usleep(10*1000);
        }
        if(read_value(pin1) == LOW)
        {
            printf("the device light cup is positive.\n");
            printf("the device active now.\n");
            set_value(pin2, HIGH);
            usleep(10*1000);
        }
        usleep(1000);
    }
}

void eaidk310_sensor_IRemission(int pin)
{
    gpio_init(pin, OUTPUT);
    for(int i=0;i<EAIDK310_LOOP*1000;i++){
        set_value(pin, HIGH);
//        if(read_value(pin) == LOW)
//        {
//            printf("the device IR emission is negative.\n");
//        }
//        if(read_value(pin) == HIGH)
//        {
//            printf("the device IR emission is positive.\n");
//            printf("the device active now.\n");
//        }
        usleep(1000);
    }
}

void eaidk310_sensor_tap_module(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin) == HIGH)
        {
            printf("the device tap module is negative.\n");
        }
        if(read_value(pin) == LOW)
        {
            printf("the device tap module is positive.\n");
            printf("the device active now.\n");
        }
        usleep(10*1000);
    }
}

void eaidk310_sensor_light_block(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin) == LOW)
        {
            printf("the device light block is negative.\n");
        }
        if(read_value(pin) == HIGH)
        {
            printf("the device light block is positive.\n");
            printf("the device active now.\n");
        }
        usleep(10*1000);
    }
}

void eaidk310_sensor_tracking(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin) == HIGH)
        {
            printf("the device track is negative.\n");
        }
        if(read_value(pin) == LOW)
        {
            printf("the device track is positive.\n");
            printf("the device active now.\n");
        }
        usleep(10*1000);
    }
}

void eaidk310_sensor_avoid(int pin)
{
    gpio_init(pin, INPUT);
    for(int i=0;i<EAIDK310_LOOP*100;i++){
        if(read_value(pin) == HIGH)
        {
            printf("the device avoid is negative.\n");
        }
        if(read_value(pin) == LOW)
        {
            printf("the device avoid is positive.\n");
            printf("the device active now.\n");
        }
        usleep(10*1000);
    }
}


#define CW_START_CLK 0
#define CW_START_DT 0
#define CW_END_CLK 1
#define CW_END_DT 1

#define CCW_START_CLK 0
#define CCW_START_DT 1
#define CCW_END_CLK 1
#define CCW_END_DT 0

void eaidk310_sensor_rotary_encode(int pin, int pin2, int pin3)
{
    gpio_init(pin, INPUT);
    gpio_init(pin2, INPUT);
    gpio_init(pin3, INPUT);
    static int step_right[3] = {0};
    static int last_right_status = 0;
    static int step_left[3] = {0};
    static int last_left_status = 0;
//
    for(int i=0;i<EAIDK310_LOOP*100;i++){
//        if(read_value(pin) == LOW)
//        {
//            printf("the device Rotary encoders is negative.\n");
//        }
//        if(read_value(pin) == HIGH)
//        {
//            printf("the device Rotary encoders is positive.\n");
//            printf("the device active now.\n");
//        }
//        usleep(500*1000);

        int dt = read_value(pin2);
        int clk =read_value(pin3);
        int sw =read_value(pin);
        printf("sw is %d, clk is %d, and dt is %d\n", sw, clk, dt);
        if(clk == CW_START_CLK && dt == CW_START_DT){ // 0 0
            if(step_right[0] && step_right[1] && last_right_status ==2){
                step_right[2] = 1;
                last_right_status =3;
            }
            if(step_right[0] && step_right[1] && step_right[2] && last_right_status == 3){
                printf("the device rotary is turnning right.................................................\n");
                step_right[0] = step_right[1] = step_right[2] = 0;
                last_right_status = 0;
            }

            //left
            if(step_left[0] && step_left[1] && last_left_status ==2){
                step_left[2] =1;
                last_left_status = 3;
            }
            if(step_left[0] && step_left[1] && step_left[2] && last_left_status ==3){
                printf("the device rotary is turnning left xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n");
                step_left[0] = step_left[1] = step_left[2] = 0;
                last_left_status = 0;
            }
        }else if(clk == CW_END_CLK && dt == CW_END_DT){ //1 1
            step_right[0] = 1;
            last_right_status = 1;

            //left
            step_left[0] =1;
            last_left_status = 1;

        }else if(clk == CW_START_CLK && dt == CCW_START_DT){ //0 1
            if(step_right[0] && step_right[1] && last_right_status ==2){
                step_right[2] = 1;
                last_right_status =3;
            }
            if(step_right[0] && step_right[1] && step_right[2] && last_right_status == 3){
                printf("the device rotary is turnning right.................................................\n");
                step_right[0] = step_right[1] = step_right[2] = 0;
                last_right_status = 0;
            }

            //left
            if(step_left[0] && last_left_status ==1)
            {
                step_left[1] =1;
                last_left_status = 2;
            }
        }else if(clk == CCW_END_CLK && dt == CCW_END_DT){ //1 0
            if(step_right[0] && last_right_status == 1){
                step_right[1] = 1;
                last_right_status =2;
            }

            //left
            if(step_left[0] && step_left[1] && last_left_status ==2){
                step_left[2] =1;
                last_left_status = 3;
            }
            if(step_left[0] && step_left[1] && step_left[2] && last_left_status ==3){
                printf("the device rotary is turnning left xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n");
                step_left[0] = step_left[1] = step_left[2] = 0;
                last_left_status = 0;
            }
        }

        usleep(10*1000);
    }
}


int pcf8591_arduino_demo()
{
    int i;
    char send[1];
    unsigned char recive[64] = {0};
    int fd;
    fd = i2c_open(I2C_FILE_NAME);
    if (fd == -1 ){
        printf("i2c open error.\n");
        return -1;
    }
    if(ioctl(fd, I2C_SLAVE, PCF8591_ADDR) < 0)
    {
        printf("Failed to connect to the sensor\n");
        return -1;
    }
    if (ioctl(fd, I2C_TIMEOUT, 1) <0)
    {
        printf("Failed to connect to the sensor\n");
        return -1;
    }
    if (ioctl(fd, I2C_RETRIES, 1) <0)
    {
        printf("Failed to connect to the sensor\n");
        return -1;
    }
    while(1)
    {
//        myAnalogRead(fd);

        memset(recive, 0x00, sizeof(recive));
        if (i2c_read_bytes(fd, 0x40, recive, sizeof(recive))<0)
        {
            continue;
        }
        printf("recive:");
        for (int i = 0;i<sizeof(recive) && recive[i] != 255;i++){
            //printf("%d ",recive[i]*100/1024); //sound
//            printf("%f ",(float)((recive[i]-143)-32)/1.8); //temp 18b20 (1)

            printf("receive is %d\n", recive[i]);
//            double temp = recive[i]*0.0048828125 * 100;
//            printf("temp is %lf\n", temp);

            //analog temp (1)
            double temp;
            temp = log((10240000/270)-10000);
            printf("temp1 is %lf\n", temp);
            temp = 1 / (0.001129148 + (0.000234125 +(0.0000000876741* temp * temp))*temp);
            printf("temp2 is %lf\n", temp);
            temp = temp - 273.15;
            printf("temp is %lf\n", temp);

        }
        printf("\n");

        usleep(1000*500);
    }
}
